package speiger.src.collections.doubles.functions.function;

import java.util.function.BiFunction;

/**
 * A Type Specific Unary Operator to support Compute/Merge functions with type specific methods to reduce boxing/unboxing
 */
public interface DoubleFloatUnaryOperator extends BiFunction<Double, Float, Float>
{
	/**
	 * A Type Specifc apply method to reduce boxing/unboxing.
	 * Applies this function to the given arguments.
	 *
	 * @param k the first function argument
	 * @param v the second function argument
	 * @return the function result
	 */
	public float applyAsFloat(double k, float v);
	
	@Override
	public default Float apply(Double k, Float v) { return Float.valueOf(applyAsFloat(k.doubleValue(), v.floatValue())); }
}