package speiger.src.collections.objects.functions.function;

import java.util.function.BiFunction;

/**
 * A Type Specific Unary Operator to support Compute/Merge functions with type specific methods to reduce boxing/unboxing
 * @param <T> the type of elements maintained by this Collection
 */
public interface ObjectFloatUnaryOperator<T> extends BiFunction<T, Float, Float>
{
	/**
	 * A Type Specifc apply method to reduce boxing/unboxing.
	 * Applies this function to the given arguments.
	 *
	 * @param k the first function argument
	 * @param v the second function argument
	 * @return the function result
	 */
	public float applyAsFloat(T k, float v);
	
	@Override
	public default Float apply(T k, Float v) { return Float.valueOf(applyAsFloat(k, v.floatValue())); }
}