package speiger.src.collections.objects.functions.function;

import java.util.function.BiFunction;

/**
 * A Type Specific Unary Operator to support Compute/Merge functions with type specific methods to reduce boxing/unboxing
 * @param <T> the type of elements maintained by this Collection
 */
public interface ObjectLongUnaryOperator<T> extends BiFunction<T, Long, Long>
{
	/**
	 * A Type Specifc apply method to reduce boxing/unboxing.
	 * Applies this function to the given arguments.
	 *
	 * @param k the first function argument
	 * @param v the second function argument
	 * @return the function result
	 */
	public long applyAsLong(T k, long v);
	
	@Override
	public default Long apply(T k, Long v) { return Long.valueOf(applyAsLong(k, v.longValue())); }
}